--
-- ahwiFM700
-- special functions for Ahwi FM700 V4.
--
-- author: Patar (Thomas Heck)
-- date: 03.01.15
--
-- extended V3: 01.09.15/Patar
-- V4 (conversion to FS17):  02.11.16/Patar

ahwiFM700 = {};

function ahwiFM700.prerequisitesPresent(specializations)
    return true;
end;

function ahwiFM700:preLoad(savegame)
	-- register new ParticleSystem for FS17 to use woodChips-spreading with smoke
	--MaterialUtil.registerParticleType("AHWI_FM700_SMOKE");
end;

function ahwiFM700:load(savegame)

-- StumpColliMovement (SCM)
    self.SCM = {
				valueY = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.stumpColli#valueY"), 1),
				valueX = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.stumpColli#valueX"), -1),
				index = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.stumpColli#index")),
				};
				
    self.maxCutTime = 6000;
-- StumpColliMovement (SCM)

-- DustParticles (DP)
    self.DP = {PS={}, free=false};

	local i = 0;
	while true do
		local key = string.format("vehicle.dustParticleSystem.emitterShape(%d)", i);
		if not hasXMLProperty(self.xmlFile, key) then
			break;
		end;
		local emitterShape = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key.."#node"));
		if emitterShape ~= nil then
			local particleSystem = MaterialUtil.getParticleSystem(FillUtil.FILLTYPE_WOODCHIPS, getXMLString(self.xmlFile, key.."#particleType"));
			if particleSystem ~= nil then
				table.insert(self.DP.PS, ParticleUtil.copyParticleSystem(self.xmlFile, key, particleSystem, emitterShape));
			end;
		end;
		i = i + 1;
	end;
-- DustParticles (DP)

-- PloughingExtension (PE)
    self.workAreas[1].disableBackwards = false;
    self.PE = {
				PS={},
				maxResetTime=1000,
				resetTime=0,
				isActive=false,
				node = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.ploughPSManipulator#node")),
				refNode = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.ploughPSManipulator#refNode")),
				rotOffset = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.ploughPSManipulator#rotOffset"), 0),
				flapWarning = {
								overlay = Overlay:new("flapWarning", Utils.getFilename("shared/smallUpDown.dds", self.baseDirectory), 0.475, 0.8, 0.1, 0.1*g_screenAspectRatio),
								active = true,
								timer = 0,
								timerMax = 700,
								};
				};


	local i = 0;
	while true do
		local key = string.format("vehicle.ploughParticleSystem.emitterShape(%d)", i);
		if not hasXMLProperty(self.xmlFile, key) then
			break;
		end;
		local emitterShape = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key.."#node"));
		local particleType = getXMLString(self.xmlFile, key.."#particleType");
		if emitterShape ~= nil then
			local fillType = FillUtil.FILLTYPE_UNKNOWN;
			local particleSystem = MaterialUtil.getParticleSystem(fillType, particleType);
			if particleSystem ~= nil then
				table.insert(self.PE.PS, ParticleUtil.copyParticleSystem(self.xmlFile, key, particleSystem, emitterShape));
			end;
		end;
		i = i + 1;
	end;
-- PloughingExtension (PE)

-- MouseControlLimit (MCL)
    self.MCL = {};
    self.MCL.backupBig = {
                            rotMax = self.movingTools[2].rotMax;
                            rotMin = self.movingTools[2].rotMin;
                            };
    self.MCL.backupSmall = {
                            rotMax = self.movingTools[1].rotMax;
                            rotMin = self.movingTools[1].rotMin;
                            };
-- MouseControlLimit (MCL)

-- ShredEverything (SE)
    self.SE = {};
    self.ahwiSplitShapeCallback = ahwiFM700.ahwiSplitShapeCallback;
-- ShredEverything (SE)

end;

function ahwiFM700:delete()
	ParticleUtil.deleteParticleSystems(self.DP.PS);
    ParticleUtil.deleteParticleSystems(self.PE.PS);
	self.PE.flapWarning.overlay:delete();
	--MaterialUtil.cleanup()
end;

function ahwiFM700:mouseEvent(posX, posY, isDown, isUp, button)
end;

function ahwiFM700:keyEvent(unicode, sym, modifier, isDown)
end;

function ahwiFM700:update(dt)
-- PloughingExtension (PE)
    if (self.isClient and not self.isServer and self.ploughHasGroundContact and self:getLastSpeed() > 3 and g_currentMission:getHasPermission("createFields", self:getOwner()) and not self.ploughLimitToField) or (self.isServer and self.lastPloughArea > 0) then
        self.PE.resetTime = 0;
    end;
    self.PE.resetTime = self.PE.resetTime + dt;
    self.PE.isActive = self.PE.resetTime < self.PE.maxResetTime;
-- PloughingExtension (PE)

end;

function ahwiFM700:updateTick(dt)

-- StumpColliMovement (SCM)
    if not self.stumpCutterParticleSystems[1].isEmitting then
        setTranslation(self.SCM.index, 0, self.SCM.valueY, 0);
    else
        local actX = (1-self.nextCutTime/self.maxCutTime)*self.SCM.valueX;
        setTranslation(self.SCM.index, actX, 0, 0);
    end;
-- StumpColliMovement (SCM)

-- DustParticles (DP)
	for k, v in pairs(self.DP.PS) do 
		ParticleUtil.setEmittingState(v, (self.resetCutTime > 0 and self.stumpCutterParticleSystems[1].isEmitting and self:getIsTurnedOn()) or (self.ploughHasGroundContact and self:getIsTurnedOn()));
	end;
-- DustParticles (DP)

-- PloughingExtension (PE)
	for k, v in pairs(self.PE.PS) do
		ParticleUtil.setEmittingState(v, self.ploughHasGroundContact and self:getIsTurnedOn() and self.PE.isActive);
	end;
	
    local x, y, z = getRotation(self.PE.refNode);
    local x = math.min(z+math.rad(self.PE.rotOffset), 0);
    setRotation(self.PE.node, x, 0, 0);
	
    if self.PE.flapWarning.timer > self.PE.flapWarning.timerMax then
        self.PE.flapWarning.active = not self.PE.flapWarning.active;
        self.PE.flapWarning.timer = 0;
    end;
    self.PE.flapWarning.timer = self.PE.flapWarning.timer + dt;
-- PloughingExtension (PE)

-- MouseControlLimit (MCL)
    if self.movingTools[2].curRot[3] < -0.619 then
        self.movingTools[1].rotMax = self.movingTools[1].rotMin+0.005;
    else
        self.movingTools[1].rotMax = self.MCL.backupSmall.rotMax;
    end;
    if self.movingTools[1].curRot[3] > self.MCL.backupSmall.rotMin+0.01 then
        self.movingTools[2].rotMin = -0.619;
    else
        self.movingTools[2].rotMin = self.MCL.backupBig.rotMin;
    end;
-- MouseControlLimit (MCL)

-- ShredEverything (SE)
    if not self.isSecondLine and self.isServer then
        if self.curSplitShape ~= nil then
            x,y,z = getWorldTranslation(self.stumpCutterCutNode);
            nx,ny,nz = localDirectionToWorld(self.stumpCutterCutNode, 1,0,0);
            yx,yy,yz = localDirectionToWorld(self.stumpCutterCutNode, 0,1,0);
            lenBelow, lenAbove = getSplitShapePlaneExtents(self.curSplitShape, x,y,z, nx,ny,nz);
        end;
        if lenBelow ~= nil then
            if lenAbove > 0.17 and self.curSplitShape ~= nil then
                splitShape(self.curSplitShape, x,y,z, nx,ny,nz, yx,yy,yz, self.stumpCutterCutSizeY, self.stumpCutterCutSizeZ, "ahwiSplitShapeCallback", self);
                self.curSplitShape = nil;
            end;
        end;
    end;
-- ShredEverything (SE)

end;

function ahwiFM700:draw()

-- PloughingExtension (PE)
    local x, _, _ = getRotation(self.PE.node);
    if self.ploughHasGroundContact and self:getIsTurnedOn() and self.PE.isActive and x == 0 and self.PE.flapWarning.active then
        self.PE.flapWarning.overlay:render();
    end;
-- PloughingExtension (PE)

-- MouseControlLimit (MCL)
    if (InputBinding.getInputAxis(InputBinding.AXIS_FRONTLOADER_ARM) > 0 or InputBinding.getIsInputMouseAxisTriggered(InputBinding.AXIS_FRONTLOADER_ARM)) and self.movingTools[2].curRot[3] == -0.619 then
        setTextColor(1,0.1,0.1,1); setTextBold(true);
        renderText(0.28, 0.86, 0.03, g_i18n:getText("MCL_pleaseLower"))
        setTextColor(1,1,1,1); setTextBold(false);
    end;
-- MouseControlLimit (MCL)

end;

function ahwiFM700:onEnter()
end;

function ahwiFM700:ahwiSplitShapeCallback(shape, isBelow, isAbove, minY, maxY, minZ, maxZ)
    if not isBelow then
        self:crushSplitShape(shape);
    end;
end;
